/*
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


// addin.cpp : Implementation of Caddin
#include "stdafx.h"
#include "OutlookAddin.h"
#include "addin.h"


#include "spdm/DMTreeFactory.h"
#include "Client/DMTClientConfig.h"
#include "base/util/utils.h"

#define APPLICATION_URI TEXT("Funambol")
#define INSTALL_DIR     T("installDir")
#define PROGRAM_NAME    TEXT("Funambol Outlook Plug-in.exe")
#define CONTEXT         T("Funambol/SyncClientOutlook")
#define APP_NAME        "Funambol Outlook Plug-in"
#define TOOLTIP         OLESTR("Synchronize")
#define CAPTION         OLESTR("Funambol Outlook Plug-in")
#define FUN             "FUN"
#define OLESTR_FUN      OLESTR("FUN")
#define WINDOW_NAME     TEXT("Funambol Outlook Plug-in")
#define SYNCHRONIZATION OLESTR("Synchronize")
#define FUNAMBOL        OLESTR("Funambol")
#define AMP_FUNAMBOL    TEXT("&Funambol")
#define CONFIGURATION   OLESTR("Configuration")
#define OUTLOOK         TEXT(" outlook")  // note the space before outlook. It's important 'cause it's used as command line argument too.

// Still used to remove Sync4j menu that could be left for error...
#define AMP_SYNC4J      TEXT("&Sync4j")
#define CAPTION_S4J     OLESTR("Sync4j Outlook Client")

// This is the path in win registry to store Addin state (installed-installing...)
#define STATE_CONTEXT   T("Microsoft/Office/Outlook/Addins/Funambol Outlook Addin")
// name of reg key
#define ADDIN_STATE     T("State")
// Possible addin state:
#define INSTALLED       T("installed")      // -> state 0
#define INSTALLING      T("installing")     // -> state 1
#define UNINSTALLING    T("uninstalling")   // -> state 2
#define UNINSTALLED     T("uninstalled")    // -> state 3


STARTUPINFO         si;
PROCESS_INFORMATION pi;
DWORD   processId;


CComQIPtr <Office::_CommandBarButton> s4jPtr;
CComQIPtr <Office::_CommandBarButton> s4jPtrItem;
CComQIPtr <Office::_CommandBarButton> s4jPtrConfig;

/////////////////////////////////////////////////////////////////////////////
// Caddin
_ATL_FUNC_INFO OnClickButtonInfo ={CC_STDCALL,VT_EMPTY,2,{VT_DISPATCH,VT_BYREF | VT_BOOL}};
_ATL_FUNC_INFO OnClickSynchronizationInfo ={CC_STDCALL,VT_EMPTY,2,{VT_DISPATCH,VT_BYREF | VT_BOOL}};
_ATL_FUNC_INFO OnClickConfigurationInfo ={CC_STDCALL,VT_EMPTY,2,{VT_DISPATCH,VT_BYREF | VT_BOOL}};

STDMETHODIMP Caddin::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] =
	{
		&IID_Iaddin
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}


STDMETHODIMP Caddin::OnConnection(IDispatch * Application, ext_ConnectMode ConnectMode, IDispatch * AddInInst, SAFEARRAY * * custom) {

     m_pParentApp = Application;

     if (ConnectMode != ext_cm_Startup)
        OnStartupComplete(custom);

    return S_OK;
}


void copyIconIntoClip() {
        HICON hIco =(HICON)::LoadImage(_Module.GetResourceInstance(),
        MAKEINTRESOURCE(IDI_ICON1),IMAGE_ICON,0,0,LR_LOADTRANSPARENT |  LR_LOADMAP3DCOLORS);
        ICONINFO oIconInfo;
        GetIconInfo(hIco, &oIconInfo);

        // put Icon into Clipboard
        ::OpenClipboard(NULL);
        ::EmptyClipboard();
        ::SetClipboardData(CF_BITMAP, oIconInfo.hbmColor);
        ::CloseClipboard();
        ::DeleteObject(hIco);

}



STDMETHODIMP Caddin::OnStartupComplete(LPSAFEARRAY* custom) {

    _ExplorerPtr            spExplorer;
    _CommandBarsPtr         spCmdBars;
    CommandBarPtr           spCmdBar;
    CommandBarControlPtr    commandBarControlPtr;

    try {
        CComVariant vEmpty(DISP_E_PARAMNOTFOUND, VT_ERROR);
        _ApplicationPtr spApp(m_pParentApp);

        spApp->ActiveExplorer(&spExplorer);
        applicationPtr = spApp;

        // Get CommandBars
	    HRESULT hRes = spExplorer->get_CommandBars(&spCmdBars);
        if(FAILED(hRes)) {
            return hRes;
        }


        // Get Addin State from win registry
        int addinState = getAddinState();

        // Check current addin State:
        // --------------------------
        //  0 -> installed      -> nothing to do
        //  1 -> installing     -> remove (old if exist) addin
        //  2 -> uninstalling   -> remove addin, unreg dll, state = 3 -> exit
        //  3 -> uninstalled    -> remove addin, unreg dll -> exit
        //                         (this case should not happen...)
        // -1 -> reg not found  -> remove addin, unreg dll, state = 3 -> exit
        //
        switch (addinState) {
            case 0:
                {
                    break;
                }
            case 1:
                {
                    removeAddin();
                    break;
                }
            case 2:
                {
                    removeAddin();
                    hRes = DllUnregisterServer();
                    setAddinState(3);
                    return hRes;
                }
            case 3:
                {
                    removeAddin();
                    hRes = DllUnregisterServer();
                    return hRes;
                }
            default:
                {
                    wchar_t msg[200];
                    wsprintf(msg, TEXT("Funambol Outlook Plug-in could not be loaded.\nPlease reinstall application."));
                    MessageBox (NULL, msg, TEXT("Funambol Outlook Addin"), MB_SETFOREGROUND |MB_OK);
                    removeAddin();
                    hRes = DllUnregisterServer();
                    setAddinState(3);
                    return hRes;
                }
        }



        //
        // Verify if CommandBar already exists
        //
	    if (SUCCEEDED(spCmdBars->get_Item(CComVariant(APP_NAME),&spCmdBar))) {

		    commandBarControlPtr = spCmdBar->FindControl(CComVariant(msoControlButton),
                                                         vEmpty,
                                                         CComVariant(FUN),
                                                         VARIANT_FALSE,
                                                         VARIANT_FALSE);

		    _CommandBarButtonPtr _commandBarButtonPtr(commandBarControlPtr);
            s4jPtr = _commandBarButtonPtr;
            s4jPtr->Enabled = true;
        }

        //
        // Add a new CommandBar to Outlook
        //
        else {
	        CommandBarPtr spNewCmdBar;
            CComPtr <Office::CommandBarControls> spBarControls;
            CComPtr <Office::CommandBarControl>  spNewBar;
            CComPtr <Office::CommandBarControl>  spNewBarConfig;
            CComVariant  vName(APP_NAME);
	        CComVariant  vPos(1);
	        CComVariant  vTemp(VARIANT_FALSE);

	        // Add a new toolband through Add method
	        spNewCmdBar = spCmdBars->Add(vName, vPos, vEmpty, vTemp);
	        spNewCmdBar->Protection = msoBarNoCustomize;

	        //now get the toolband's CommandBarControls
	        spBarControls = spNewCmdBar->GetControls();
            CComVariant vToolBarType(msoControlButton);

	        //show the toolbar?
	        CComVariant vShow(VARIANT_FALSE);

            // Add first button
	        spNewBar = spBarControls->Add(vToolBarType, vEmpty, vEmpty, vEmpty, vShow);

            // get CommandBarButton interface for each toolbar button
	        // so we can specify button styles and stuff
	        // each button displays a bitmap and caption next to it
            CComQIPtr <Office::_CommandBarButton> spCmdButton(spNewBar);
            // to set a bitmap to a button, load a 32x32 bitmap
            // and copy it to clipboard. Call CommandBarButton's PasteFace()
            // to copy the bitmap to the button face. to use
            // Outlook's set of predefined bitmap, set button's FaceId to //the
            // button whose bitmap you want to use

            //
            // Modified to load an ICON
            //
            spCmdButton->BeginGroup = true;
            spCmdButton->PutStyle(msoButtonIcon);
            copyIconIntoClip();

            // set style before setting bitmap
            hRes = spCmdButton->PasteFace();
            if (hRes < 0) {
	            spCmdButton->PutFaceId(1758);
            }

	        spCmdButton->PutVisible(VARIANT_TRUE);
	        spCmdButton->PutCaption(CAPTION);
	        spCmdButton->PutEnabled(VARIANT_TRUE);
	        spCmdButton->PutTooltipText(TOOLTIP);
	        spCmdButton->PutTag(OLESTR_FUN);

            s4jPtr = spCmdButton;
	        spNewCmdBar->PutVisible(VARIANT_TRUE);
            s4jPtr->Enabled = true;
        }


        //
        // Verify if Funambol menu exists
        //
        CComPtr <Office::CommandBarControls> spCmdCtrls;
        CComPtr <Office::CommandBarControl> spCmdCtrl;

        spCmdCtrls = spCmdBars->ActiveMenuBar->GetControls();
        ATLASSERT(spCmdCtrls);

        hRes = spCmdCtrls->get_Item(CComVariant(AMP_FUNAMBOL), &spCmdCtrl);
        // Add new menu bar
        if (FAILED(hRes)) {
            AddNewMenubar(spCmdBars);
        }
        else {
            CommandBarPopupPtr pMenuItem;
            pMenuItem = spCmdBars->ActiveMenuBar->Controls->GetItem(AMP_FUNAMBOL);
            s4jPtrItem   = pMenuItem->Controls->GetItem(SYNCHRONIZATION);
            s4jPtrConfig = pMenuItem->Controls->GetItem(CONFIGURATION);
        }

        DispAdviseControls();

        //
        // If here, Funambol Addin is installed correctly -> state = 0 ("installed")
        //
        setAddinState(0);
    }


    catch(_com_error &e) {
        //wchar_t msg[200];
        //wsprintf(msg, TEXT("Unexpected error occurred: \"%ls\"\nPlease reinstall Funambol Outlook Plug-in."), e.ErrorMessage());
        //MessageBox (NULL, msg, TEXT("Funambol Outlook Addin"), MB_SETFOREGROUND |MB_OK);
        return S_FALSE;
    }

	return S_OK;
}



STDMETHODIMP Caddin::OnDisconnection(ext_DisconnectMode RemoveMode, SAFEARRAY * * custom)
{

    try {
        HRESULT hRes = 0;

        // not used...
        if (RemoveMode == 1) {
            CComVariant vEmpty(DISP_E_PARAMNOTFOUND, VT_ERROR);
            _ExplorerPtr    spExplorer;
            CommandBarPtr commandBarPtr;
            applicationPtr->ActiveExplorer(&spExplorer);

            _CommandBarsPtr spCmdBars;

            hRes = spExplorer->get_CommandBars(&spCmdBars);
            if(FAILED(hRes)) {
                return hRes;
            }

            hRes = spCmdBars->get_Item(CComVariant(APP_NAME),&commandBarPtr);
            if (SUCCEEDED(hRes))
            {
	            // get the Button
	            CommandBarControlPtr commandBarControlPtr;
	            commandBarControlPtr = commandBarPtr->FindControl(CComVariant(msoControlButton),vEmpty,CComVariant(FUN),VARIANT_FALSE,VARIANT_FALSE);
	            if (commandBarControlPtr != NULL)
	            {
		            commandBarControlPtr->Delete();
	            }
	            commandBarPtr->Delete();
            }
            DeleteMenubar(spCmdBars);
        }

        // hRes = DispEventUnadvise((IDispatch*)s4jPtr);
        hRes = DispUnadviseControls();
        if(FAILED(hRes)) {
            return hRes;
        }
    }

    catch(_com_error &e) {
        //wchar_t msg[200];
        //wsprintf(msg, TEXT("Unexpected error occurred: \"%ls\"\nPlease reinstall Funambol Outlook Plug-in."), e.ErrorMessage());
        //MessageBox (NULL, msg, TEXT("Funambol Outlook Addin"), MB_SETFOREGROUND |MB_OK);
        return S_FALSE;
    }

	return S_OK;
}

STDMETHODIMP Caddin::OnBeginShutdown(SAFEARRAY ** custom) {

    return S_OK;
}


void __stdcall Caddin::OnClickSynchronization(IDispatch * /*Office::_CommandBarButton**/ Ctrl,VARIANT_BOOL * CancelDefault) {

    launchSyncClientOutlook(OUTLOOK);
}

void __stdcall Caddin::OnClickConfiguration(IDispatch * /*Office::_CommandBarButton**/ Ctrl,VARIANT_BOOL * CancelDefault) {

    launchSyncClientOutlook(NULL);
}



//---------------------------------------------------------------------------
void __stdcall Caddin::OnClickButton(IDispatch* /*Office::_CommandBarButton* */ Ctrl, VARIANT_BOOL * CancelDefault)
{

    //
    // Test if arleady exists a window named "SyncClient Outlook" with the same process id.
    // If there is, put in foreground else create a new one
    //
    launchSyncClientOutlook(OUTLOOK);

}

/*
* If paramenter equals "outlook" then the method was lauched by the Configuration Item
* If it has "outlook" the button or the item that start sync in background
*/

void Caddin::launchSyncClientOutlook(wchar_t* parameter) {

    HWND hwnd = FindWindow(NULL, WINDOW_NAME);
    if (hwnd != NULL) {

        HANDLE h = OpenProcess(PROCESS_ALL_ACCESS, FALSE, processId);
        if (h != NULL) {
            SetForegroundWindow(hwnd);
            CloseHandle(h);
            CloseHandle(hwnd);
            return;
        }
        CloseHandle(hwnd);
    }

    char* dir = new char[512];
    wchar_t* wdir;
    wchar_t program [512];


	DMTree* dmt = DMTreeFactory::getDMTree(CONTEXT);
    ManagementNode *node = dmt->getManagementNode(CONTEXT);
    dir = node->getPropertyValue(INSTALL_DIR);

    wdir = toWideChar(dir);
    wsprintf(program, TEXT("%ls\\%ls"), wdir, PROGRAM_NAME);


    //
    // Add the command line "outlook" to the SyncClientOutlook.exe
    //
    if (parameter != NULL && wcscmp(parameter, OUTLOOK) == 0) {
        wcscat(program, OUTLOOK);
    }

    ZeroMemory( &si, sizeof(si) );
    si.cb = sizeof(si);
    ZeroMemory( &pi, sizeof(pi) );
    BOOL res = FALSE;
    // Start the child process.

    SetCurrentDirectory(wdir);
    res = CreateProcess( NULL,   // No module name (use command line).
        program,
        NULL,             // Process handle not inheritable.
        NULL,             // Thread handle not inheritable.
        FALSE,            // Set handle inheritance to FALSE.
        0,                // No creation flags.
        NULL,             // Use parent's environment block.
        NULL,             // Use parent's starting directory.
        &si,              // Pointer to STARTUPINFO structure.
        &pi );            // Pointer to PROCESS_INFORMATION structure.

    processId = pi.dwProcessId;

    // Close process and thread handles.
    CloseHandle( pi.hProcess );
    CloseHandle( pi.hThread );

    if (dmt != NULL)
        delete dmt;
    if (node != NULL)
        delete node;
    if (dir)
        delete [] dir;
    if (wdir)
        delete [] wdir;

}


// not used...
HRESULT Caddin::DeleteMenubar(_CommandBars *pCmdBars) {

    CComPtr < Office::CommandBarControls> spCmdCtrls;
    CComPtr < Office::CommandBarControl> spCmdCtrl;

    spCmdCtrls = pCmdBars->ActiveMenuBar->GetControls();
    ATLASSERT(spCmdCtrls);

    HRESULT hr = spCmdCtrls->get_Item(CComVariant(AMP_FUNAMBOL), &spCmdCtrl);

    if (SUCCEEDED(hr)) {
        CommandBarPopupPtr pMenuItem;
        pMenuItem = pCmdBars->ActiveMenuBar->Controls->GetItem(AMP_FUNAMBOL);
        pMenuItem->Delete(vtMissing);

    }

    return S_OK;
}


HRESULT Caddin::AddNewMenubar(_CommandBars *pCmdBars)
{
    CComPtr < Office::CommandBar > spNewCmdBar;
    HRESULT hr = pCmdBars->get_ActiveMenuBar(&spNewCmdBar);
    if (FAILED(hr))
        return hr;

    CComPtr< Office::CommandBarControls > spCtrls;
    spCtrls = spNewCmdBar->GetControls();
    ATLASSERT(spCtrls);

    CComVariant varType(10);
    CComVariant vtEmpty(DISP_E_PARAMNOTFOUND, VT_ERROR);
    //CComVariant vtId(1001); // id
    CComVariant vtTemp(VARIANT_FALSE);
    //CComVariant vtBefore(7);
    CComPtr< CommandBarControl > spCtrl;
    spCtrl = spCtrls->Add(varType, vtEmpty, vtEmpty, vtEmpty, vtTemp);

    ATLASSERT(spCtrl);

    CComQIPtr< Office::CommandBarPopup > spPopup(spCtrl->GetControl());
    ATLASSERT(spPopup);

    // set menu caption
    spPopup->PutCaption(FUNAMBOL);
    spPopup->PutVisible(VARIANT_TRUE);

    // now add a menu item to the menubar
    // as a CommandBarButton so you can specify styles

    CComPtr< Office::CommandBarControls > spCmdBarCtrls;
    spCmdBarCtrls = spPopup->GetControls();

    // A menu is a CommandBarPopup
    // object(MsoControlType::msoControlPopup = 10),
    // to which we have to add our menuitems i.e.
    // CommandBarButton(MsoControlType::msoControlButton = 1).
    // you can also add popup menus similarly.

    CComVariant vType(1); // button

    //
    // Button Synchronization
    //

    CComPtr< Office::CommandBarControl > spBarCtrl;

    spBarCtrl = spCmdBarCtrls->Add(vType, vtEmpty, vtEmpty, vtEmpty, vtTemp);
    ATLASSERT(spBarCtrl);

    CComQIPtr< Office::_CommandBarButton > spButton(spBarCtrl);
    ATLASSERT(spButton);

    // set button styles
    spButton->PutCaption(SYNCHRONIZATION);
    copyIconIntoClip();

    spButton->PasteFace();
    spButton->PutStyle(Office::msoButtonIconAndCaption);
    spButton->PutVisible(VARIANT_TRUE);

    s4jPtrItem = spButton;

    //
    // Button configuration
    //

    CComPtr< Office::CommandBarControl > spBarCtrlConfig;

    spBarCtrlConfig = spCmdBarCtrls->Add(vType, vtEmpty, vtEmpty, vtEmpty, vtTemp);
    ATLASSERT(spBarCtrl);

    CComQIPtr< Office::_CommandBarButton > spButtonConfig(spBarCtrlConfig);
    ATLASSERT(spButtonConfig);

    // set button styles
    spButtonConfig->PutCaption(CONFIGURATION);
    //spButtonConfig->PutFaceId(0);
    //spButtonConfig->PutStyle(Office::msoButtonIconAndCaption);
    spButtonConfig->PutStyle(Office::msoButtonCaption);
    spButtonConfig->PutVisible(VARIANT_TRUE);

    s4jPtrConfig = spButtonConfig;

    return S_OK;
}

HRESULT __stdcall Caddin::DispAdviseControls(void)
{
	HRESULT hr = ButtonSyncEvent::DispEventAdvise((IDispatch*)s4jPtr,&DIID__CommandBarButtonEvents);
	if (FAILED(hr))
		return hr;

    if (s4jPtrItem != NULL) {
        hr = ItemSynchronizationEvent::DispEventAdvise((IDispatch*)s4jPtrItem,&DIID__CommandBarButtonEvents);
	    if (FAILED(hr))
		    return hr;
    }

    if (s4jPtrConfig != NULL) {
        hr = ItemConfigurationEvent::DispEventAdvise((IDispatch*)s4jPtrConfig,&DIID__CommandBarButtonEvents);
	    if (FAILED(hr))
		    return hr;
    }

	return S_OK;
}

HRESULT __stdcall Caddin::DispUnadviseControls(void)
{

	HRESULT hr = ButtonSyncEvent::DispEventUnadvise((IDispatch*)s4jPtr);
    if(FAILED(hr)) {
        return hr;
    }
    if (s4jPtrItem != NULL) {
	    hr = ItemSynchronizationEvent::DispEventUnadvise((IDispatch*)s4jPtrItem);
        if(FAILED(hr)) {
		    return hr;
        }
    }
    if (s4jPtrConfig != NULL) {
        hr = ItemConfigurationEvent::DispEventUnadvise((IDispatch*)s4jPtrConfig);
        if(FAILED(hr)) {
		    return hr;
        }
    }
    return S_OK;
}



/*
 * Removes the Funambol Outlook Addin from Outlook.
 * 1. Remove Funambol CommandBar (if exists)
 * 2. Remove Funambol menu named "Funambol" (if exists)
 */
HRESULT Caddin::removeAddin() {

    _ExplorerPtr             spExplorer;
    _CommandBarsPtr          spCmdBars;
    CommandBarControlPtr     commandBarControlPtr;
    CommandBarPtr            spCmdBar;
    CommandBarPopupPtr       pMenuItem;
    CommandBarControlsPtr    spCmdCtrls;
    CommandBarControlPtr     spCmdCtrl;

    VARIANT variant;
	variant.vt = VT_BSTR;
    HRESULT hRes;

    try {
        CComVariant vEmpty(DISP_E_PARAMNOTFOUND, VT_ERROR);
        _ApplicationPtr spApp(m_pParentApp);

        // Get Outlook Command Bars
        spApp->ActiveExplorer(&spExplorer);
	    hRes = spExplorer->get_CommandBars(&spCmdBars);
        if(FAILED(hRes)) {
            return hRes;
        }

        //
        // Get and remove Funambol Command Bar
        //
        variant.bstrVal = SysAllocString(CAPTION);
        hRes = spCmdBars->get_Item(variant, &spCmdBar);
        //VariantClear(&variant);
        if (SUCCEEDED(hRes)) {
            hRes = spCmdBar->Delete();
        }


        // get CommandBar that is Outlook's main menu
        hRes = spCmdBars->get_ActiveMenuBar(&spCmdBar);
        if (FAILED(hRes)) {
            return hRes;
        }
        spCmdCtrls = spCmdBar->GetControls();
        ATLASSERT(spCmdCtrls);

        //
        // Get and remove Funambol menu
        //
        hRes = spCmdCtrls->get_Item(CComVariant(AMP_FUNAMBOL), &spCmdCtrl);
        if (SUCCEEDED(hRes)) {
            pMenuItem = spCmdBars->ActiveMenuBar->Controls->GetItem(AMP_FUNAMBOL);
            pMenuItem->Delete(vtMissing);
        }


        //
        // Also get and remove Sync4j bar/menu that could be left :)
        //
        variant.bstrVal = SysAllocString(CAPTION_S4J);
        hRes = spCmdBars->get_Item(variant, &spCmdBar);
        VariantClear(&variant);
        if (SUCCEEDED(hRes)) {
            hRes = spCmdBar->Delete();
        }
        hRes = spCmdCtrls->get_Item(CComVariant(AMP_SYNC4J), &spCmdCtrl);
        if (SUCCEEDED(hRes)) {
            pMenuItem = spCmdBars->ActiveMenuBar->Controls->GetItem(AMP_SYNC4J);
            pMenuItem->Delete(vtMissing);
        }
    }

    catch(_com_error &e) {
        //wchar_t msg[200];
        //wsprintf(msg, TEXT("Unexpected error occurred while removing Funambol Outlook Addin.\n\"%s\""), e.ErrorMessage());
        //MessageBox(NULL, msg, L"Funambol Outlook Addin Remover", MB_SETFOREGROUND |MB_OK);
        return S_FALSE;
    }

    return S_OK;
}


/*
 * Get addin state from win registry (key = STATE_CONTEXT)
 * "installed"     -> return 0
 * "installing"    -> return 1
 * "uninstalling"  -> return 2
 * "uninstalled"   -> return 3
 * not found       -> return -1
 */
int Caddin::getAddinState() {

    int value = -1;
    char* state = new char[20];

	DMTree* dmt = DMTreeFactory::getDMTree(STATE_CONTEXT);
    ManagementNode *node = dmt->getManagementNode(STATE_CONTEXT);
    if (!node) {
        // node not found
        return -1;
    }
    state = node->getPropertyValue(ADDIN_STATE);

    if (!strcmp(state, INSTALLED)) {
        value = 0;
    }
    else if (!strcmp(state, INSTALLING)) {
        value = 1;
    }
    else if (!strcmp(state, UNINSTALLING)) {
        value = 2;
    }
    else if (!strcmp(state, UNINSTALLED)) {
        value = 3;
    }
    else {
        // key not found
        value = -1;
    }

    if (state) {
        delete [] state; state = NULL;
    }
    return value;
}


/*
 * Save addin state in win registry (key = STATE_CONTEXT)
 */
int Caddin::setAddinState(int value) {

    char* state = new char[20];

    if (value == 0) {
        sprintf(state, INSTALLED);
    }
    else if (value == 1) {
        sprintf(state, INSTALLING);
    }
    else if (value == 2) {
        sprintf(state, UNINSTALLING);
    }
    else if (value == 3) {
        sprintf(state, UNINSTALLED);
    }
    else {
        // This should not happen...
        sprintf(state, INSTALLING);
        return -1;
    }

	DMTree* dmt = DMTreeFactory::getDMTree(STATE_CONTEXT);
    ManagementNode *node = dmt->getManagementNode(STATE_CONTEXT);
    if (!node) {
        // node not found
        return -1;
    }

    node->setPropertyValue(ADDIN_STATE, state);

    if (state) {
        delete [] state; state = NULL;
    }
    return 0;
}




