Attribute VB_Name = "mdFileSystem"
'
' Copyright (C) 2003-2006 Funambol
'
' This program is free software; you can redistribute it and/or modify
' it under the terms of the GNU General Public License as published by
' the Free Software Foundation; either version 2 of the License, or
' (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'

'****************************************************************
'*Module:  mdFileSystem.bas                                     *
'*                                                              *
'*This module contains all the functions for controlling files  *
'*and folders in the file system, reading data from user files, *
'*loading default data if needed, and delete the unused files   *
'*at the end of the synchronization.                            *
'****************************************************************

Option Explicit

Function doesFileExist(speciFile As String) As Boolean
    'function to test if file specified in speciFile is a valid file.
    'Return True if file specified in the speciFile argument exists.
    'Return False if speciFile is not a valid file or if speciFile is a directory.

On Error GoTo doesFileExist_Err

If (GetAttr(speciFile) And vbDirectory) <> vbDirectory Then
    doesFileExist = CBool(Len(Dir(speciFile)) > 0)
Else
    doesFileExist = False
End If

doesFileExist_Exit:
    Exit Function

doesFileExist_Err:
    doesFileExist = False
    Resume doesFileExist_Exit

End Function

Function doesFolderExist(speciFolder As String) As Boolean
    'function to test if folder specified in speciFolder is a valid folder.
    'Return True if folder specified in the speciFolder argument exists.
    'Return False if speciFolder is not a valid folder or if speciFolder is a file.

On Error GoTo doesFolderExist_Err

If (GetAttr(speciFolder) And vbDirectory) = vbDirectory Then
    doesFolderExist = True
Else
    doesFolderExist = False
End If

doesFolderExist_Exit:
    Exit Function

doesFolderExist_Err:
    doesFolderExist = False
    Resume doesFolderExist_Exit

End Function

Public Sub readAllItemPropertiesForUser(ByRef nameOfFile As String)
    'read the Item Properties list from the file nameOfFile
    'and save the data into itemProperties array

Dim readingLIne As String
Dim readingArray() As String
Dim i As Integer
Dim commentHere As String

On Error GoTo readAllItemPropertiesForUser_Err

i = 0 'variable to count how many lines saved in the array itemProperties

'test if the file exists before opening it
If Not doesFileExist(nameOfFile) Then Exit Sub

'open the file for reading data
Open nameOfFile For Input As #1
Do While Not EOF(1)
    Line Input #1, readingLIne 'read line by line
    If Len(readingLIne) > 0 Then
        'cut the spaces at the beginning of the line
        Do While Left(readingLIne, 1) = " "
            readingLIne = Right(readingLIne, Len(readingLIne) - 1)
        Loop
        'search for the comment lines or comment characters
        commentHere = InStr(readingLIne, "||")
        'if the comment is not at the beginning of the line
        If commentHere >= 1 Then
            'cut the comment part from a string
            readingLIne = Left(readingLIne, commentHere - 1)
        End If
        'if in the string there is the TAB character
        If Len(readingLIne) > 0 And InStr(readingLIne, Chr(9)) Then
            'to divide the string in fields
            readingArray = Split(readingLIne, Chr(9))
            'the 5th position is for the USE column in LIST_FOR_PROPERTIES.db file
            If Len(readingArray(5)) > 0 Then
                If InStr(readingArray(5), "Yes") Then
                    mdVariables.itemProperties(i).PropID = readingArray(0)
                    mdVariables.itemProperties(i).Property = readingArray(1)
                    mdVariables.itemProperties(i).DataType = readingArray(2)
                    mdVariables.itemProperties(i).name = readingArray(4)
                    If readingArray(6) = "False" Then
                        mdVariables.itemProperties(i).InCal = False
                    ElseIf readingArray(6) = "True" Then
                        mdVariables.itemProperties(i).InCal = True
                    End If
                    If readingArray(7) = "False" Then
                        mdVariables.itemProperties(i).InCon = False
                    ElseIf readingArray(7) = "True" Then
                        mdVariables.itemProperties(i).InCon = True
                    End If
                    If readingArray(8) = "False" Then
                        mdVariables.itemProperties(i).InMail = False
                    ElseIf readingArray(8) = "True" Then
                        mdVariables.itemProperties(i).InMail = True
                    End If
                    If readingArray(9) = "False" Then
                        mdVariables.itemProperties(i).InTask = False
                    ElseIf readingArray(9) = "True" Then
                        mdVariables.itemProperties(i).InTask = True
                    End If
                    If readingArray(10) = "False" Then
                        mdVariables.itemProperties(i).InNote = False
                    ElseIf readingArray(10) = "True" Then
                        mdVariables.itemProperties(i).InNote = True
                    End If
                    If readingArray(11) = "No" Then
                        mdVariables.itemProperties(i).IsDefault = False
                    ElseIf readingArray(11) = "Yes" Then
                        mdVariables.itemProperties(i).IsDefault = True
                    End If
                    i = i + 1
                End If
            End If
        End If
    End If
Loop
Close #1

'redimension the array using only the lines needed
ReDim Preserve mdVariables.itemProperties(0 To i - 1)

readAllItemPropertiesForUser_Exit:
    Exit Sub

readAllItemPropertiesForUser_Err:

    Dim actionToDo As Integer
    
    actionToDo = mdErrors.fixErrors(Err.number)
    Select Case actionToDo
    Case 0
        GoTo readAllItemPropertiesForUser_Exit
    Case 1
        Resume
    Case 2
        Resume Next
    End Select

End Sub

Sub readPreferencesFromFile(ByRef filename As String)
    'read the preferences saved in the file fileName
    'and save the data into the correct array

Dim readingLIne As String
Dim readingArray() As String
Dim commentHere As Integer

On Error GoTo readPreferencesFromFile_Err

'to count how many selected folders in preferences file
mdVariables.countSelected = 0

Open filename For Input As #1
Do While Not EOF(1)
    Line Input #1, readingLIne
    If Len(readingLIne) > 0 Then
        'cut the spaces at the beginning of the line
        Do While Left(readingLIne, 1) = " "
            readingLIne = Right(readingLIne, Len(readingLIne) - 1)
        Loop
        'search for the comment lines
        commentHere = InStr(readingLIne, "||")
        'if the comment is not at the beginning of the line
        If commentHere > 1 Then
            'cut the comment part from a string
            readingLIne = Left(readingLIne, commentHere - 1)
        End If
        If Len(readingLIne) > 0 And InStr(readingLIne, "=") Then
            'the first split divides the property-head of the line from the content
            readingArray = Split(readingLIne, "=")
            If Len(readingArray(1)) > 0 Then 'if the property-list part of the line is not empty
                If InStr(readingArray(0), "folder") Then 'Case "folder" written in the property-head part of line
                    'complete the reading of the property list based on the type
                    If InStr(readingArray(0), "Calendar") Then
                        completeFolderList readingArray(1), mdVariables.FOLDER_TYPE_CAL
                    ElseIf InStr(readingArray(0), "Contacts") Then
                        completeFolderList readingArray(1), mdVariables.FOLDER_TYPE_CON
                    ElseIf InStr(readingArray(0), "Mail") Then
                        completeFolderList readingArray(1), mdVariables.FOLDER_TYPE_MAIL
                    ElseIf InStr(readingArray(0), "Tasks") Then
                        completeFolderList readingArray(1), mdVariables.FOLDER_TYPE_TASK
                    ElseIf InStr(readingArray(0), "Notes") Then
                        completeFolderList readingArray(1), mdVariables.FOLDER_TYPE_NOTE
                    End If
                ElseIf InStr(readingArray(0), "properties") Then  'Case "properties" written in the property-head part of line
                    'complete the reading of the property list based on the type
                    If InStr(readingArray(0), "Calendar") Then
                        fillItemPropertiesList readingArray(1), mdVariables.FOLDER_TYPE_CAL
                    ElseIf InStr(readingArray(0), "Contacts") Then
                        fillItemPropertiesList readingArray(1), mdVariables.FOLDER_TYPE_CON
                    ElseIf InStr(readingArray(0), "Mail") Then
                        fillItemPropertiesList readingArray(1), mdVariables.FOLDER_TYPE_MAIL
                    ElseIf InStr(readingArray(0), "Tasks") Then
                        fillItemPropertiesList readingArray(1), mdVariables.FOLDER_TYPE_TASK
                    ElseIf InStr(readingArray(0), "Notes") Then
                        fillItemPropertiesList readingArray(1), mdVariables.FOLDER_TYPE_NOTE
                    End If
                End If
            End If
        End If
    End If
Loop
Close #1

readPreferencesFromFile_Exit:
    Exit Sub

readPreferencesFromFile_Err:

    Dim actionToDo As Integer
    
    actionToDo = mdErrors.fixErrors(Err.number)
    Select Case actionToDo
    Case 0
        GoTo readPreferencesFromFile_Exit
    Case 1
        Resume
    Case 2
        Resume Next
    End Select

End Sub

Sub completeFolderList(ByRef readingFolders As String, ByRef folderType As Integer)
    'read from readingFolders string a list of EntryID values for the selected folders to sync

Dim readingArray() As String
Dim i As Integer, j As Integer

On Error GoTo completeFolderList_Err

readingArray = Split(readingFolders, ",") 'create the EntryID array for the selected folders

Select Case folderType
'set in calendarFolders the selection property for the selected Calendar folders
Case mdVariables.FOLDER_TYPE_CAL
    For i = 0 To UBound(readingArray)
        logToDebug "readingArray(I):" & readingArray(i)
        For j = 1 To UBound(mdVariables.calendarFolders)
            If mdVariables.calendarFolders(j).EntryID = readingArray(i) Then
                mdVariables.calendarFolders(j).Selected = True
                mdVariables.countSelected = mdVariables.countSelected + 1
                Exit For
            End If
        Next j
    Next i
'set in contactsFolders the selection property for the selected Contacts folders
Case mdVariables.FOLDER_TYPE_CON
    For i = 0 To UBound(readingArray)
        logToDebug "readingArray(I):" & readingArray(i)
        For j = 1 To UBound(mdVariables.contactsFolders)
            If mdVariables.contactsFolders(j).EntryID = readingArray(i) Then
                mdVariables.contactsFolders(j).Selected = True
                mdVariables.countSelected = mdVariables.countSelected + 1
                Exit For
            End If
        Next j
    Next i
'set in mailFolders the selection property for the selected Mail folders
Case mdVariables.FOLDER_TYPE_MAIL
    For i = 0 To UBound(readingArray)
        logToDebug "readingArray(I):" & readingArray(i)
        For j = 1 To UBound(mdVariables.mailFolders)
            If mdVariables.mailFolders(j).EntryID = readingArray(i) Then
                mdVariables.mailFolders(j).Selected = True
                mdVariables.countSelected = mdVariables.countSelected + 1
                Exit For
            End If
        Next j
    Next i
'set in tasksFolders the selection property for the selected Tasks folders
Case mdVariables.FOLDER_TYPE_TASK
    For i = 0 To UBound(readingArray)
        logToDebug "readingArray(I):" & readingArray(i)
        For j = 1 To UBound(mdVariables.tasksFolders)
            If mdVariables.tasksFolders(j).EntryID = readingArray(i) Then
                mdVariables.tasksFolders(j).Selected = True
                mdVariables.countSelected = mdVariables.countSelected + 1
                Exit For
            End If
        Next j
    Next i
'set in notesFolders the selection property for the selected Notes folders
Case mdVariables.FOLDER_TYPE_NOTE
    For i = 0 To UBound(readingArray)
        logToDebug "readingArray(I):" & readingArray(i)
        For j = 1 To UBound(mdVariables.notesFolders)
            If mdVariables.notesFolders(j).EntryID = readingArray(i) Then
                mdVariables.notesFolders(j).Selected = True
                mdVariables.countSelected = mdVariables.countSelected + 1
                Exit For
            End If
        Next j
    Next i
End Select

logToDebug mdVariables.countSelected & " selected folders"

completeFolderList_Exit:
    Exit Sub

completeFolderList_Err:

    Dim actionToDo As Integer
    
    actionToDo = mdErrors.fixErrors(Err.number)
    Select Case actionToDo
    Case 0
        GoTo completeFolderList_Exit
    Case 1
        Resume
    Case 2
        Resume Next
    End Select

End Sub

Sub fillItemPropertiesList(ByRef readingProperties As String, ByRef folderType As Integer)
    'read from readingProperties string a list of propID values for the selected Item Properties to sync

Dim readingArray() As String
Dim i As Integer
Dim j As Integer

On Error GoTo fillItemPropertiesList_Err

readingArray = Split(readingProperties, ",") 'create the Properties array for the items type

Select Case folderType
'set in calendarFolders the selection property for the selected Calendar folders
Case mdVariables.FOLDER_TYPE_CAL
    ReDim mdVariables.calendarProperties(UBound(readingArray) + 1)
    For i = 1 To UBound(mdVariables.calendarProperties)
        logToDebug "readingArray" & i - 1 & ":" & readingArray(i - 1)
        mdVariables.calendarProperties(i).PropID = readingArray(i - 1)
        For j = 0 To UBound(mdVariables.itemProperties)
            If mdVariables.itemProperties(j).PropID = Val(readingArray(i - 1)) Then
                mdVariables.calendarProperties(i).name = mdVariables.itemProperties(j).name
                mdVariables.calendarProperties(i).Property = mdVariables.itemProperties(j).Property
                Exit For
            End If
        Next j
    Next i
'set in contactsFolders the selection property for the selected Contacts folders
Case mdVariables.FOLDER_TYPE_CON
    ReDim mdVariables.contactsProperties(UBound(readingArray) + 1)
    For i = 1 To UBound(mdVariables.contactsProperties)
        logToDebug "readingArray" & i - 1 & ":" & readingArray(i - 1)
        mdVariables.contactsProperties(i).PropID = readingArray(i - 1)
        For j = 0 To UBound(mdVariables.itemProperties)
            If mdVariables.itemProperties(j).PropID = Val(readingArray(i - 1)) Then
                mdVariables.contactsProperties(i).name = mdVariables.itemProperties(j).name
                mdVariables.contactsProperties(i).Property = mdVariables.itemProperties(j).Property
                Exit For
            End If
        Next j
    Next i
'set in mailFolders the selection property for the selected Mail folders
Case mdVariables.FOLDER_TYPE_MAIL
    ReDim mdVariables.mailProperties(UBound(readingArray) + 1)
    For i = 1 To UBound(mdVariables.mailProperties)
        logToDebug "readingArray" & i - 1 & ":" & readingArray(i - 1)
        mdVariables.mailProperties(i).PropID = readingArray(i - 1)
        For j = 0 To UBound(mdVariables.itemProperties)
            If mdVariables.itemProperties(j).PropID = Val(readingArray(i - 1)) Then
                mdVariables.mailProperties(i).name = mdVariables.itemProperties(j).name
                mdVariables.mailProperties(i).Property = mdVariables.itemProperties(j).Property
                Exit For
            End If
        Next j
    Next i
'set in tasksFolders the selection property for the selected Tasks folders
Case mdVariables.FOLDER_TYPE_TASK
    ReDim mdVariables.tasksProperties(UBound(readingArray) + 1)
    For i = 1 To UBound(mdVariables.tasksProperties)
        logToDebug "readingArray" & i - 1 & ":" & readingArray(i - 1)
        mdVariables.tasksProperties(i).PropID = readingArray(i - 1)
        For j = 0 To UBound(mdVariables.itemProperties)
            If mdVariables.itemProperties(j).PropID = Val(readingArray(i - 1)) Then
                mdVariables.tasksProperties(i).name = mdVariables.itemProperties(j).name
                mdVariables.tasksProperties(i).Property = mdVariables.itemProperties(j).Property
                Exit For
            End If
        Next j
    Next i
'set in notesFolders the selection property for the selected Notes folders
Case mdVariables.FOLDER_TYPE_NOTE
    ReDim mdVariables.notesProperties(UBound(readingArray) + 1)
    For i = 1 To UBound(mdVariables.notesProperties)
        logToDebug "readingArray" & i - 1 & ":" & readingArray(i - 1)
        mdVariables.notesProperties(i).PropID = readingArray(i - 1)
        For j = 0 To UBound(mdVariables.itemProperties)
            If mdVariables.itemProperties(j).PropID = Val(readingArray(i - 1)) Then
                mdVariables.notesProperties(i).name = mdVariables.itemProperties(j).name
                mdVariables.notesProperties(i).Property = mdVariables.itemProperties(j).Property
                Exit For
            End If
        Next j
    Next i
End Select

fillItemPropertiesList_Exit:
    Exit Sub

fillItemPropertiesList_Err:

    Dim actionToDo As Integer
    
    actionToDo = mdErrors.fixErrors(Err.number)
    Select Case actionToDo
    Case 0
        GoTo fillItemPropertiesList_Exit
    Case 1
        Resume
    Case 2
        Resume Next
    End Select

End Sub

Public Sub loadDefaultFolders()
    'load in the arrays for selected folders the values for the default folders

Dim i As Integer
Dim stringCalFolder As String
Dim stringConFolder As String
Dim stringMailFolder As String
Dim stringTaskFolder As String
Dim stringNoteFolder As String

On Error GoTo loadDefaultFolders_Err

'select default folders, start counting from 1, because in 0 there is no folder name saved
For i = 1 To UBound(mdVariables.calendarFolders)
    'test if the folder is the default one for this type
    If mdVariables.calendarFolders(i).name = nameSpace.GetDefaultFolder(olFolderCalendar).name Then
        'if yes fix the folder as selected and changed
        If Not mdVariables.calendarFolders(i).Selected Then
            mdVariables.calendarFolders(i).Selected = True
            mdVariables.calendarFolders(i).Changed = True
        End If
    Else
        'if no, delete the old selected folder if exists
        If mdFileSystem.doesFolderExist(mdVariables.calendarFolders(i).EntryID) Then
            ChDir mdVariables.calendarFolders(i).EntryID
            Kill "*.*"
            ChDir ".."
            RmDir mdVariables.calendarFolders(i).EntryID 'delete the relative folder if exsists
        End If
        mdVariables.calendarFolders(i).Changed = False
        mdVariables.calendarFolders(i).Selected = False
    End If
Next i
For i = 1 To UBound(mdVariables.contactsFolders)
    'test if the folder is the default one for this type
    If mdVariables.contactsFolders(i).name = nameSpace.GetDefaultFolder(olFolderContacts).name Then
        'if yes fix the folder as selected and changed
        If Not mdVariables.contactsFolders(i).Selected Then
            mdVariables.contactsFolders(i).Selected = True
            mdVariables.contactsFolders(i).Changed = True
        End If
    Else
        'if no, delete the old selected folder if exists
        If mdFileSystem.doesFolderExist(mdVariables.contactsFolders(i).EntryID) Then
            ChDir mdVariables.contactsFolders(i).EntryID
            Kill "*.*"
            ChDir ".."
            RmDir mdVariables.contactsFolders(i).EntryID 'delete the relative folder if exsists
        End If
        'mdVariables.contactsFolders(I).Changed = False
        'mdVariables.contactsFolders(I).Selected = False
        'MM
        mdVariables.contactsFolders(i).Changed = True
        mdVariables.contactsFolders(i).Selected = True
    End If
Next i
For i = 1 To UBound(mdVariables.mailFolders)
    'test if the folder is the default one for this type
    If mdVariables.mailFolders(i).name = nameSpace.GetDefaultFolder(olFolderInbox).name Then
        'if yes fix the folder as selected and changed
        If Not mdVariables.mailFolders(i).Selected Then
            mdVariables.mailFolders(i).Selected = True
            mdVariables.mailFolders(i).Changed = True
        End If
    Else
        'if no, delete the old selected folder if exists
        If mdFileSystem.doesFolderExist(mdVariables.mailFolders(i).EntryID) Then
            ChDir mdVariables.mailFolders(i).EntryID
            Kill "*.*"
            ChDir ".."
            RmDir mdVariables.mailFolders(i).EntryID 'delete the relative folder if exsists
        End If
        mdVariables.mailFolders(i).Changed = False
        mdVariables.mailFolders(i).Selected = False
    End If
Next i
For i = 1 To UBound(mdVariables.tasksFolders)
    'test if the folder is the default one for this type
    If mdVariables.tasksFolders(i).name = nameSpace.GetDefaultFolder(olFolderTasks).name Then
        'if yes fix the folder as selected and changed
        If Not mdVariables.tasksFolders(i).Selected Then
            mdVariables.tasksFolders(i).Selected = True
            mdVariables.tasksFolders(i).Changed = True
        End If
    Else
        'if no, delete the old selected folder if exists
        If mdFileSystem.doesFolderExist(mdVariables.tasksFolders(i).EntryID) Then
            ChDir mdVariables.tasksFolders(i).EntryID
            Kill "*.*"
            ChDir ".."
            RmDir mdVariables.tasksFolders(i).EntryID 'delete the relative folder if exsists
        End If
        mdVariables.tasksFolders(i).Changed = False
        mdVariables.tasksFolders(i).Selected = False
    End If
Next i
For i = 1 To UBound(mdVariables.notesFolders)
    'test if the folder is the default one for this type
    If mdVariables.notesFolders(i).name = nameSpace.GetDefaultFolder(olFolderNotes).name Then
        'if yes fix the folder as selected and changed
        If Not mdVariables.notesFolders(i).Selected Then
            mdVariables.notesFolders(i).Selected = True
            mdVariables.notesFolders(i).Changed = True
        End If
    Else
        'if no, delete the old selected folder if exists
        If mdFileSystem.doesFolderExist(mdVariables.notesFolders(i).EntryID) Then
            ChDir mdVariables.notesFolders(i).EntryID
            Kill "*.*"
            ChDir ".."
            RmDir mdVariables.notesFolders(i).EntryID 'delete the relative folder if exsists
        End If
        mdVariables.notesFolders(i).Changed = False
        mdVariables.notesFolders(i).Selected = False
    End If
Next i

'prepare the strings to write the default folders EntryIDs to the preferences file
stringCalFolder = "Calendar_folders_to_sync=" & nameSpace.GetDefaultFolder(olFolderCalendar).EntryID
stringConFolder = "Contacts_folders_to_sync=" & nameSpace.GetDefaultFolder(olFolderContacts).EntryID
stringMailFolder = "Mail_folders_to_sync=" & nameSpace.GetDefaultFolder(olFolderInbox).EntryID
stringTaskFolder = "Tasks_folders_to_sync=" & nameSpace.GetDefaultFolder(olFolderTasks).EntryID
stringNoteFolder = "Notes_folders_to_sync=" & nameSpace.GetDefaultFolder(olFolderNotes).EntryID

'MM to keep all the
stringConFolder = "Contacts_folders_to_sync="
For i = 1 To UBound(mdVariables.contactsFolders)
    If mdVariables.contactsFolders(i).Selected Then
        stringConFolder = stringConFolder & mdVariables.contactsFolders(i).EntryID & ","
    End If
Next i
'if the last character is a comma, delete it
If Right(stringConFolder, 1) = "," Then
    stringConFolder = Left(stringConFolder, Len(stringConFolder) - 1)
End If


'save the new selected folders in fhe file
Open mdVariables.NAME_PREFERENCES_FILE For Output As #1

Print #1, stringCalFolder
Print #1, stringConFolder
Print #1, stringMailFolder
Print #1, stringTaskFolder
Print #1, stringNoteFolder

Close #1

loadDefaultFolders_Exit:
    Exit Sub

loadDefaultFolders_Err:

    Dim actionToDo As Integer
    
    actionToDo = mdErrors.fixErrors(Err.number)
    Select Case actionToDo
    Case 0
        GoTo loadDefaultFolders_Exit
    Case 1
        Resume
    Case 2
        Resume Next
    End Select
End Sub

Public Sub loadDefaultProperties()
    'load the default Item Properties reading from itemProperties array

Dim i As Integer
Dim stringCalItemProp As String
Dim stringConItemProp As String
Dim stringMailItemProp As String
Dim stringTaskItemProp As String
Dim stringNoteItemProp As String

On Error GoTo loadDefaultProperties_Err

    'read the list of all item properties searching the default selections
    'in order to save them in the default file
    
    'prepare the string to write in the items.preferences file
    stringCalItemProp = "Calendar_item_properties_to_sync="
    stringConItemProp = "Contacts_item_properties_to_sync="
    stringMailItemProp = "Mail_item_properties_to_sync="
    stringTaskItemProp = "Tasks_item_properties_to_sync="
    stringNoteItemProp = "Notes_item_properties_to_sync="

    For i = 0 To UBound(mdVariables.itemProperties)
        'test the type of the item in order to copy
        'the default property in the correct array
        If mdVariables.itemProperties(i).IsDefault Then
            If mdVariables.itemProperties(i).InCal Then 'Calendar type
                'complete the string to write in the items.preferences file
                stringCalItemProp = stringCalItemProp & mdVariables.itemProperties(i).PropID & ","
            End If
            If mdVariables.itemProperties(i).InCon Then 'Contacts type
                'complete the string to write in the items.preferences file
                stringConItemProp = stringConItemProp & mdVariables.itemProperties(i).PropID & ","
            End If
            If mdVariables.itemProperties(i).InMail Then 'Mail type
                'complete the string to write in the items.preferences file
                stringMailItemProp = stringMailItemProp & mdVariables.itemProperties(i).PropID & ","
            End If
            If mdVariables.itemProperties(i).InTask Then 'Tasks type
                'complete the string to write in the items.preferences file
                stringTaskItemProp = stringTaskItemProp & mdVariables.itemProperties(i).PropID & ","
            End If
            If mdVariables.itemProperties(i).InNote Then 'Notes type
                'complete the string to write in the items.preferences file
                stringNoteItemProp = stringNoteItemProp & mdVariables.itemProperties(i).PropID & ","
            End If
        End If
    Next i

    'if the last character is a comma, delete it from all the 4 strings
    If Right(stringCalItemProp, 1) = "," Then
        stringCalItemProp = Left(stringCalItemProp, Len(stringCalItemProp) - 1)
    End If
    If Right(stringConItemProp, 1) = "," Then
        stringConItemProp = Left(stringConItemProp, Len(stringConItemProp) - 1)
    End If
    If Right(stringMailItemProp, 1) = "," Then
        stringMailItemProp = Left(stringMailItemProp, Len(stringMailItemProp) - 1)
    End If
    If Right(stringTaskItemProp, 1) = "," Then
        stringTaskItemProp = Left(stringTaskItemProp, Len(stringTaskItemProp) - 1)
    End If
    If Right(stringNoteItemProp, 1) = "," Then
        stringNoteItemProp = Left(stringNoteItemProp, Len(stringNoteItemProp) - 1)
    End If

    'save the new selected item properties in fhe file
    Open mdVariables.NAME_PROPERTIES_FILE For Output As #1

    Print #1, stringCalItemProp
    Print #1, stringConItemProp
    Print #1, stringMailItemProp
    Print #1, stringTaskItemProp
    Print #1, stringNoteItemProp

    Close #1

'Re-test if the file now exists
If mdFileSystem.doesFileExist(mdVariables.NAME_PROPERTIES_FILE) Then
    mdVariables.propExists = True
    'read the preferences from the default file
    mdFileSystem.readPreferencesFromFile mdVariables.NAME_PROPERTIES_FILE
Else
    mdVariables.propExists = False
End If

loadDefaultProperties_Exit:
    Exit Sub

loadDefaultProperties_Err:

    Dim actionToDo As Integer
    
    actionToDo = mdErrors.fixErrors(Err.number)
    Select Case actionToDo
    Case 0
        GoTo loadDefaultProperties_Exit
    Case 1
        Resume
    Case 2
        Resume Next
    End Select

End Sub

Public Sub endOfSync()
    'after receiving sync complete confirmation, loops through
    'all the disk folders for items files updating the Old file
    'with the All file and deleting all the files but Old one

Dim i As Integer

On Error GoTo endOfSync_Err

'For I = 0 To UBound(mdVariables.selectedFolders)
'    ChDir mdVariables.selectedFolders(I).EntryID
'
'    If doesFileExist(mdVariables.NAME_CRE_FILE) = True Then
'        'MM
'        listAllItemsInFolder mdVariables.nameSpace.GetFolderFromID( _
'            mdVariables.selectedFolders(I).EntryID), mdVariables.selectedFolders(I).Type
'
'        Kill mdVariables.NAME_CRE_FILE
'        Kill mdVariables.NAME_MOD_FILE
'        Kill mdVariables.NAME_DEL_FILE
'        Kill mdVariables.NAME_OLD_FILE
'        Name mdVariables.NAME_ALL_FILE As mdVariables.NAME_OLD_FILE
'    End If
'    ChDir ".."
'Next I

endOfSync_Exit:
    Exit Sub

endOfSync_Err:

    Dim actionToDo As Integer
    
    actionToDo = mdErrors.fixErrors(Err.number)
    Select Case actionToDo
    Case 0
        GoTo endOfSync_Exit
    Case 1
        Resume
    Case 2
        Resume Next
    End Select

End Sub


Public Function checkHttpErrors(ByVal errorCode As Integer) As Boolean

    checkHttpErrors = False
    If (errorCode >= 2000 And errorCode < 3000) Then
        checkHttpErrors = True
        'MsgBox ("Connection failure.")
    End If

End Function


Public Sub endOfFolderSync(ByVal olFolderType As Integer, ByVal folderType As Integer, ByVal success As Boolean)
    'MM. I f success is True then sync successfully terminated. Else errors happens
    
On Error GoTo endOfSync_Err
    printDebugMessage "Enter endOfFolderSync method"
    ChDrive (mdMMVariables.AppDrive)
    ChDir nameSpace.GetDefaultFolder(olFolderType).EntryID
        
    If success = True Then
        
        'If mdFileSystem.doesFileExist(mdVariables.NAME_CRE_FILE) = True Then
            'MM
            listAllItemsInFolder mdVariables.nameSpace.GetFolderFromID( _
                nameSpace.GetDefaultFolder(olFolderType).EntryID), folderType, nameSpace.GetDefaultFolder(olFolderType).name
                       
            On Error Resume Next
                Kill mdVariables.NAME_CRE_FILE
                Kill mdVariables.NAME_MOD_FILE
                Kill mdVariables.NAME_DEL_FILE
                Kill mdVariables.NAME_OLD_FILE
            On Error GoTo 0
            
            Name mdVariables.NAME_ALL_FILE As mdVariables.NAME_OLD_FILE
            
        'End If
       
    
    Else 'if no success don't delete file old...
        
        'If doesFileExist(mdVariables.NAME_CRE_FILE) = True Then
         On Error Resume Next
            Kill mdVariables.NAME_CRE_FILE
            Kill mdVariables.NAME_MOD_FILE
            Kill mdVariables.NAME_DEL_FILE
            Kill mdVariables.NAME_ALL_FILE
         On Error GoTo 0
        'End If
                
    End If
   
    ChDir ".."


endOfSync_Exit:
    printDebugMessage "Exit endOfFolderSync method"
    Exit Sub

endOfSync_Err:

    Dim actionToDo As Integer
    
    actionToDo = mdErrors.fixErrors(Err.number)
    Select Case actionToDo
    Case 0
        GoTo endOfSync_Exit
    Case 1
        Resume
    Case 2
        Resume Next
    End Select

End Sub


Public Sub endOfFolderSyncContacts(ByRef folderInfoArray() As FolderInfo, ByVal folderType As Integer, success As Boolean)
    'MM. I f success is True then sync successfully terminated. Else errors happens
    
printDebugMessage "Enter endOfFolderSyncContacts method"
Dim i As Long
i = 1

On Error GoTo endOfSync_Err
   
    For i = 1 To UBound(folderInfoArray)
        ChDrive (mdMMVariables.AppDrive)
        ChDir folderInfoArray(i).EntryID
        
        If success = True Then
            
            'MM
            listAllItemsInFolder mdVariables.nameSpace.GetFolderFromID(folderInfoArray(i).EntryID), folderType, folderInfoArray(i).CompletePath

            'If doesFileExist(mdVariables.NAME_CRE_FILE) = True Then
            On Error Resume Next
                Kill mdVariables.NAME_CRE_FILE
                Kill mdVariables.NAME_MOD_FILE
                Kill mdVariables.NAME_DEL_FILE
                Kill mdVariables.NAME_OLD_FILE
            On Error GoTo 0
            'End If

            Name mdVariables.NAME_ALL_FILE As mdVariables.NAME_OLD_FILE
            
        Else 'if no success don't delete file old...
            
            'If doesFileExist(mdVariables.NAME_CRE_FILE) = True Then
            On Error Resume Next
                Kill mdVariables.NAME_CRE_FILE
                Kill mdVariables.NAME_MOD_FILE
                Kill mdVariables.NAME_DEL_FILE
                Kill mdVariables.NAME_ALL_FILE
            On Error GoTo 0
            'End If
                        
        End If
        
        ChDir ".."
        
    Next i

    If success = True Then
        'delete the contacts subfolder deleted by user but
        Dim dimension As Long
        dimension = -1
On Error GoTo endOfSync_Err
        dimension = UBound(deletedFolders)
        
        
        For i = 0 To dimension
            
            If Dir(mdVariables.deletedFolders(i), vbDirectory) <> "" Then
            'If doesFolderExist(mdVariables.deletedFolders(I)) = True Then
                ChDir mdVariables.deletedFolders(i)
                
                Kill "*.*"
                            
                ChDir ".."
                RmDir mdVariables.deletedFolders(i)
            End If
            
        Next i
    Else
        ' do nothing. re-send items next time
    End If
        
    If success = True Then ' rewrite folder.preferences with all the value of the contacts properties
        'delete the contacts subfolder deleted by user but
        
        
        Dim readingLIne As String
        
        Open mdVariables.NAME_PREFERENCES_FILE For Input As #1
        Open "tmp.preferences" For Output As #2
        
        Do While Not EOF(1)
            Line Input #1, readingLIne 'read line by line
            If InStr(readingLIne, "Contacts") Then
            
                readingLIne = "Contacts_folders_to_sync="
                For i = 1 To UBound(mdVariables.contactsFolders)
                   
                    readingLIne = readingLIne & mdVariables.contactsFolders(i).EntryID & ","
                   
                Next i
                
                'if the last character is a comma, delete it
                If Right(readingLIne, 1) = "," Then
                    readingLIne = Left(readingLIne, Len(readingLIne) - 1)
                End If
                
                Print #2, readingLIne
                
            Else
                Print #2, readingLIne
            End If
            
        Loop
        
        Close #1
        Close #2
        
        Kill mdVariables.NAME_PREFERENCES_FILE
        Name "tmp.preferences" As mdVariables.NAME_PREFERENCES_FILE
        
    Else
        ' if sync is failed the new folder that aren't in the mdVariables.NAME_PREFERENCES_FILE have to be deleted
        ' do nothing. re-send items next time
        
        Dim readLine As String
        Dim positionFolder As String
        readLine = ""
        
        Open mdVariables.NAME_PREFERENCES_FILE For Input As #1
        
        Do While Not EOF(1)
            Line Input #1, readLine 'read line by line
            
            If InStr(readLine, "Contacts") Then
                Close #1
                Exit Do
            End If
            
        Loop
                        
        If readLine <> "" Then
            For i = 1 To UBound(mdVariables.contactsFolders)
                positionFolder = 0
                positionFolder = InStr(readLine, mdVariables.contactsFolders(i).EntryID)
                If positionFolder <= 0 Then
                    
                    'On Error Resume Next
                    ChDir mdVariables.contactsFolders(i).EntryID
                    Kill "*.*"
                    ChDir ".."
                    If Dir(mdVariables.contactsFolders(i).EntryID, vbDirectory) <> "" Then
                        RmDir mdVariables.contactsFolders(i).EntryID
                    End If
                    logToDebug CurDir
                    'On Error GoTo 0
                End If
               
            Next i
                            
        End If
            
        
        
                
    End If
    
    
endOfSync_Exit:
    printDebugMessage "Exit endOfFolderSyncContacts method"
    Exit Sub

endOfSync_Err:

    Dim actionToDo As Integer
    
    actionToDo = mdErrors.fixErrors(Err.number)
    Select Case actionToDo
    Case 0
        GoTo endOfSync_Exit
    Case 1
        Resume
    Case 2
        Resume Next
    End Select

End Sub

Public Function readUnicodeFile(ByVal path As String)
Dim oFS As Object
Dim oStream As Object
Dim oFIle As Object
Dim sRecord As String
Dim filen As String
sRecord = ""
filen = path + "/" + "ServerItems.txt"
   Set oFS = CreateObject("Scripting.FileSystemObject")
   Set oFIle = oFS.getFile(filen)
   Set oStream = oFIle.OpenAsTextStream(1, -1)
   sRecord = oStream.ReadAll()
    
   'Do While Not oStream.AtEndOfStream
   '   sRecord = sRecord + oStream.readLine
   'Loop
   
   oStream.Close
   readUnicodeFile = sRecord

End Function

